package cn.dankal.test.independentconsumer;

import cn.dankal.test.common.Const;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.PartitionInfo;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.Properties;
import java.util.stream.Collectors;

/**
 * 测试 Kafka : Kafka 独立消费者测试
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-27
 */
public class IndependentConsumerTest {

    /**
     * KafkaConsumer
     */
    private static KafkaConsumer<String, String> kafkaConsumer;


    @BeforeAll
    public static void before() {
        Properties properties = new Properties();
        final String bootstrapServers = "192.168.128.3:9092,192.168.128.3:9093,192.168.128.3:9094";
        // broker(Kafka服务器) 地址清单
        properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        // message key 反序列化器
        properties.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getCanonicalName());
        // message value 反序列化器
        properties.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getCanonicalName());

        // 通过 Properties 配置 实例化 KafkaConsumer
        kafkaConsumer = new KafkaConsumer<>(properties);
    }

    /**
     * Kafka 独立消费者 测试
     */
    @Test
    public void testKafkaIndependentConsumer() {
        // 获取分区信息
        List<PartitionInfo> partitionInfoList = kafkaConsumer.partitionsFor(Const.Kafka.Topic.HELLO_TOPIC);
        // 通过 PartitionInfoList  封装   topicPartitionList
        List<TopicPartition> topicPartitionList = partitionInfoList.stream()
                .map(
                        partitionInfo -> new TopicPartition(partitionInfo.topic(), partitionInfo.partition())
                )
                .collect(Collectors.toList());
        // 给独立消费者分配分区
        kafkaConsumer.assign(topicPartitionList);
        while (true) {
            ConsumerRecords<String, String> consumerRecords = kafkaConsumer.poll(Duration.ofMillis(500));
            for (ConsumerRecord<String, String> consumerRecord : consumerRecords) {
                System.out.printf(
                        "%s 消息消息 ==> 主题: %s 分区: %s 偏移量: %s key: %s value: %s %n",
                        Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                        consumerRecord.topic(),
                        consumerRecord.partition(),
                        consumerRecord.offset(),
                        consumerRecord.key(),
                        consumerRecord.value());
                // TODO  分区效果需要我们自己都过编码实现
            }
        }
    }

    @AfterAll
    public static void after() {
        if (Objects.nonNull(kafkaConsumer)) {
            kafkaConsumer.close();
        }
    }
}
