package cn.dankal.test.producersendmode;

import cn.dankal.test.common.Const;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.kafka.common.serialization.StringSerializer;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.util.Objects;
import java.util.Properties;
import java.util.concurrent.ExecutionException;

/**
 * Kafka 生产者: 生产消息的三种形式
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-26
 */
public class KafkaProducerSendModeTest {


    /**
     * KafkaProducer
     */
    private static KafkaProducer<String, String> kafkaProducer;


    @BeforeAll
    public static void before() {
        Properties properties = new Properties();
        final String bootstrapServers = "192.168.128.3:9092,192.168.128.3:9093,192.168.128.3:9094";
        // broker(Kafka服务器) 地址清单
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        // message key 序列化器
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getCanonicalName());
        // message value 序列化器
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getCanonicalName());

        // 通过 Properties 配置 实例化 KafkaProducer
        kafkaProducer = new KafkaProducer<>(properties);
    }

    /**
     * 测试: Kafka生产者发送模式: 发送并忘记
     */
    @Test
    public void testSendAndForget() {
        final String topic = Const.Kafka.Topic.HELLO_TOPIC;
        final String key = "dankal-test-send-mode";
        final String message = "dankal";
        ProducerRecord<String, String> producerRecord =
                new ProducerRecord<>(
                        topic,
                        key,
                        message
                );
        // mode: 发送并忘记
        kafkaProducer.send(producerRecord);
    }

    /**
     * 测试: Kafka生产者发送模式: 同步发送
     */
    @Test
    public void testSyncSendMessage() throws ExecutionException, InterruptedException {
        final String topic = Const.Kafka.Topic.HELLO_TOPIC;
        final String key = "dankal-test-send-mode";
        final String message = "dankal";
        ProducerRecord<String, String> producerRecord =
                new ProducerRecord<>(
                        topic,
                        key,
                        message
                );
        // mode: 同步发送
        RecordMetadata recordMetadata = kafkaProducer.send(producerRecord)
                // 调用 Future#get() 会阻塞当前线程
                .get();
        System.out.printf(
                "完成消息发送 %d ==> 主题: %s  分区: %s 偏移量: %d%n",
                recordMetadata.timestamp(),
                recordMetadata.topic(),
                recordMetadata.partition(),
                recordMetadata.offset()
        );
    }

    /**
     * 测试: Kafka生产者发送模式: 异步发送
     */
    @Test
    public void testAsyncSendMessage() {
        final String topic = Const.Kafka.Topic.HELLO_TOPIC;
        final String key = "dankal-test-send-mode";
        final String message = "dankal";
        ProducerRecord<String, String> producerRecord =
                new ProducerRecord<>(
                        topic,
                        key,
                        message
                );
        // mode: 异步发送
        kafkaProducer.send(
                producerRecord,
                (recordMetadata, exception) -> {
                    // 发送消息失败: exception message
                    if (null != exception) {
                        exception.printStackTrace();
                        return;
                    }
                    // 发送消息成功
                    System.out.printf(
                            "异步发送消息成功 %d ==> 主题: %s  分区: %s 偏移量: %d%n",
                            recordMetadata.timestamp(),
                            recordMetadata.topic(),
                            recordMetadata.partition(),
                            recordMetadata.offset());
                }
        );
        System.out.println("已经通过异步形式发送 Kafka 消息");
    }


    @AfterAll
    public static void after() {
        if (Objects.nonNull(kafkaProducer)) {
            kafkaProducer.close();
        }
    }
}
