package cn.dankal.test.rebalance;

import cn.dankal.test.common.Const;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import org.apache.kafka.clients.consumer.*;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 测试 Kafka Rebalance: ConsumerWorker
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-26
 */
public class ConsumerWorker {
    /**
     * Map<TopicPartition, OffsetAndMetadata> 当前消费者分区与偏移量信息
     */
    private final Map<TopicPartition, OffsetAndMetadata> offsetAndMetadataMap;

    /**
     * current ConsumerWorker KafkaConsumer instance
     */
    private final KafkaConsumer<String, String> kafkaConsumer;

    /**
     * 是否是临时工作者: 临时工作者在消费五条消息之后将退出当前消费群组
     */
    private final boolean isTempWorker;

    /**
     * 临时工作者的消费消息的次数: 5
     */
    private final static int TEMP_WORK_COUNT = 5;

    /**
     * KafkaConsumer config properties
     */
    private static final Properties PROPERTIES;


    //TODO maybe 事务类 提供事务支持
    //private final Transactional transactional;

    static {
        PROPERTIES = new Properties();
        final String bootstrapServers = "192.168.128.3:9092,192.168.128.3:9093,192.168.128.3:9094";
        // broker(Kafka服务器) 地址清单
        PROPERTIES.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        // message key 反序列化器
        PROPERTIES.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getCanonicalName());
        // message value 反序列化器
        PROPERTIES.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getCanonicalName());
        PROPERTIES.put(ConsumerConfig.GROUP_ID_CONFIG, Const.Kafka.ConsumerGroupId.TEST_KAFKA_REBALANCE);

    }

    public ConsumerWorker(boolean isTempWorker) {
        this.offsetAndMetadataMap = new ConcurrentHashMap<>();
        this.isTempWorker = isTempWorker;
        // 消费者线程不安全,建议一个线程一个对应一个消费者实例
        this.kafkaConsumer = new KafkaConsumer<>(PROPERTIES);

        // 设置 分区再均衡监听器
        this.kafkaConsumer.subscribe(
                CollectionUtil.newArrayList(Const.Kafka.Topic.HELLO_TOPIC),
                new RebalanceListener(offsetAndMetadataMap, kafkaConsumer)
        );
    }

    /**
     * 执行 Kafka message consumer work
     */
    public void doWork() {
        // 执行计数 ,搭配 isTempWorker 使用
        int execCount = 0;
        try {
            while (true) {
                ConsumerRecords<String, String> consumerRecords = kafkaConsumer.poll(Duration.ofMillis(500));
                for (ConsumerRecord<String, String> consumerRecord : consumerRecords) {
                    try {
                        // TODO Transaction: transaction begin
                        // transaction.begin();
                        // TODO 消费消息 处理业务
                        long currentMessageOffset = consumerRecord.offset();
                        System.out.printf(
                                "%s 消息消息 ==> 主题: %s 分区: %s 偏移量: %s key: %s value: %s %n",
                                Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                                consumerRecord.topic(),
                                consumerRecord.partition(),
                                currentMessageOffset,
                                consumerRecord.key(),
                                consumerRecord.value());
                        // TODO  保存分区与偏移量信息
                        TopicPartition topicPartition = new TopicPartition(
                                consumerRecord.topic(),
                                consumerRecord.partition()
                        );
                        OffsetAndMetadata offsetAndMetadata = new OffsetAndMetadata(currentMessageOffset + 1, "no metadata");
                        offsetAndMetadataMap.put(topicPartition, offsetAndMetadata);

                        //TODO 保存偏移量与分区信息
                        if (MapUtil.isNotEmpty(offsetAndMetadataMap)) {
                            for (TopicPartition partition : offsetAndMetadataMap.keySet()) {
                                Const.PARTITION_OFFSET_INFO_MAP.put(
                                        partition,
                                        offsetAndMetadataMap.get(partition).offset()
                                );
                            }
                        }

                        // TODO Transaction: transaction commit
                        // transaction.commit();
                        kafkaConsumer.commitAsync();

                        //TODO 执行计数
                        execCount++;

                        // TODO just test kafka Rebalance
                        if (isTempWorker) {
                            if (execCount == TEMP_WORK_COUNT) {
                                // 触发分区再均衡
                                // 临时工作者工作完成~ 即将推出 当前消费群组
                                System.out.printf(
                                        "%s 完成临时工作即将推出工作群组. offsetAndMetadataMap: %s%n",
                                        Thread.currentThread().getName(),
                                        offsetAndMetadataMap);
                                //提交自己已经完成的工作消息偏移量
                                kafkaConsumer.commitSync();
                                return;
                            }
                        }
                    } catch (Exception e) {
                        // TODO Transaction: transaction rollback
                        // transaction.rollback();

                        // TODO handler exception
                        e.printStackTrace();
                    }
                }
            }
        } finally {
            kafkaConsumer.commitSync();
            kafkaConsumer.close();
        }

    }
}
